#!/usr/bin/perl

# Paisley detects patterns in strings fed to it via either
#	 files, a string argument, or stdin.
# Use the '--help' option for usage, license, and copyright information.
# This software is copyright (c) 2007 Andrew Merenbach.

use strict;
use warnings;
#use diagnostics;

#use encoding 'utf8';

use Pod::Usage;
use Getopt::Long 2.34;

$main::VERSION = '0.1a1';

our $MAX_FILE_READ_LEN = 2**20 - 1;

# prototypes
sub main();
sub find_factors($);
sub read_file($);
sub determine_search_mode($);
sub determine_input_string(%);
sub find_pattern($);
sub find_pattern_from_end($);
sub find_pattern_from_side($$);
#sub find_pattern_from_left($);
#sub find_pattern_from_right($);

sub main() {
	my %opts = (
		'newline' => 0,	# print a newline after output
		'mode' => '',	# search mode
		'file' => '',	# should we input from filename?
		'string' => '',	# should we input from string?
	);
	
	GetOptions(
		'n|newline' => \$opts{'newline'},
		
		'm|mode=s' => \$opts{'mode'},
		'f|file=s' => \$opts{'file'},
		's|string=s' => \$opts{'string'},
		
		'help|?' => sub { pod2usage(-verbose => 2) },
	);
	
	my $search_mode = determine_search_mode($opts{'mode'});
	my $input_string = determine_input_string(\%opts);
	
	my $pattern = '';
	
	for ($search_mode) {
		/plain/ and $pattern = find_pattern($input_string), last;
		/left/ and $pattern = find_pattern_from_side($input_string, 0), last;	
		/right/ and $pattern = find_pattern_from_side($input_string, 1), last;
	}
	
	if ($pattern ne '') {
		print $pattern;
		print "\n" if ($opts{'newline'} == 1);
	}
}

sub find_factors($) {
	my $number = shift;
	
	my @factors = (1);
	my $root = sqrt $number;
	my $quotient;
	
	for (my $i = 2; $i < $root; ++$i) {
		$quotient = $number / $i;
		push(@factors, $quotient, $i) if (int $quotient == $quotient);
	}
		
	# add square root if it is an integer factor
	push (@factors, $root) if (int $root == $root);
	
	# sort ascending
	@factors = sort {$main::a <=> $main::b} @factors;
	
	return @factors;
}

sub read_file($) {
	my $filepath = shift;
	my $contents;
	
	my $handle;
	open ($handle, "<$filepath") or die "*** Can't find file \"$filepath.\"\n";
	read ($handle, $contents, $MAX_FILE_READ_LEN);
	close ($handle);
	
	return $contents;
}

sub determine_search_mode($) {
	my $opt = shift;

	my $search_mode = '';
	my $mode_count = 0;
	
	if ($opt eq 'l' or $opt eq 'left') {
		$search_mode = 'left';
		++$mode_count;
	}
	
	if ($opt eq 'r' or $opt eq 'right') {
		$search_mode = 'right';
		++$mode_count;
	}
	
	if ($opt eq 'p' or $opt eq 'plain') {
		$search_mode = 'plain';
		++$mode_count;
	}
	
	if ($mode_count > 1) {
		die "*** I cannot perform multiple types of pattern searches at once.\n";
	} elsif ($mode_count < 1) {
		die "*** You must specify some type of pattern search to be performed.\n";
	}
	
	return $search_mode;
}

sub determine_input_string(%) {
	my $opts = shift;

	my $input_string = '';
	my $input_count = 0;
	
	my $opt;
	
	$opt = $$opts{'string'};
	if ($opt ne '') {
		$input_string = $opt;
		++$input_count;
	}
	
	$opt = $$opts{'file'};
	if ($opt ne '') {
		$input_string = read_file($opt);
		++$input_count;
	}
	
	if ($input_count > 1) {
		die "*** I cannot process multiple types of input at once.\n";
	} elsif ($input_count < 1) {
		die "*** You must specify some input to process.\n";
		#$input_string = <STDIN>;
		#chomp $input_string;
	}
	
	return $input_string;
}

# find a pattern constituting a string
sub find_pattern($) {
	my $string = shift;
	my $pattern = '';
	
	my $len = length $string;
	
	if ($len > 1) {
		my $search_term;
		my $regex_str;
		my @group_sizes = find_factors($len);
		
		foreach (@group_sizes) {
			$search_term = substr $string, 0, $_;
			($regex_str = $string) =~ s/(\Q$search_term\E)*//g;
			$pattern = $search_term, last if ($regex_str eq '');
		}
	}
	
	return $pattern;
}

sub find_pattern_from_end($) {
	my $input_string = shift;
	my $output_string = '';
	
	# continue as long as the input string is not yet empty
	#	and the output pattern has not yet been determined
	while ($input_string ne '' and $output_string eq '') {
		$output_string = find_pattern($input_string);
		chop $input_string;
	}
	
	return $output_string;
}

sub find_pattern_from_side($$) {
	my ($data, $right_side) = @_;
	$data = reverse $data if ($right_side == 1);
	my $pattern = find_pattern_from_end($data);
	$pattern = reverse $pattern if ($right_side == 1);
	return $pattern;
}

# find a pattern starting on the left edge of a string
#sub find_pattern_from_left($) {
	#my $input_string = shift;
	#my $output_string = '';
	#
	#my $substring;	
	#
	#for (my $idx = length($input_string); $idx > 0; --$idx) {
	#	$substring = substr($input_string, 0, $idx);
	#	$output_string = find_pattern($substring);
	#	last if ($output_string ne '');
	#}
	#
	#return $output_string;
#}

# find a pattern ending on the right edge of a string
#sub find_pattern_from_right($) {
	#my $input_string = shift;
	#my $output_string = '';
	#
	#my $substring;	
	#my $len = length($input_string);
	#
	#for (my $idx = 0; $idx < $len; ++$idx) {
	#	$substring = substr($input_string, $idx);
	#	$output_string = find_pattern($substring);
	#	last if ($output_string ne '');
	#}
	#
	#return $output_string;
#}

main();

__END__

=head1 NAME

Paisley - the regular-expression pattern-finder

=head1 SYNOPSIS

paisley [-n] [-plr] [-s string] [-f filename]

=head1 OPTIONS

=over 8

=item -n	print newline after output

=item --mode <mode>

perform pattern search with specified mode: plain (p), left (l), or right (r).  The plain search examines the entire input string for an overall pattern; left-searching searches from left to right; and right-searching searches from right to left.

=item -m <mode>

same as --mode <mode>

=item --string <string>

string to process (mutually exclusive with -f)

=item -s <string>

same as --string <string>

=item --file <file>

file to process (mutually exclusive with -s)

=item -f <filename>

same as --file <file>

=item -?

display this help

=item --help

display this help

=back

=head1 DESCRIPTION

Paisley detects patterns in text that the user inputs from either stdin (via a pipe or post-launch input), via the '-s' string argument, or via the '-f' filename argument.
		
=head1 EXAMPLES

The following examples show how to use the software.

- To perform a left-side pattern search, use the following syntax:

	paisley --mode left --string "hellohellodolly"

- To pipe input into Paisley for processing on the right side, use:

	echo "hellodollydolly" | paisley --mode right

- To search for a pattern across a whole string, with the output followed by a newline, use the following syntax:

	paisley --mode plain --string "hellodollyhellodolly" --newline

The first of these examples will return "hello"; the second, "dolly"; and the third, "hellodolly" (followed by a newline).  The command-line arguments for 'mode,' 'newline,' and string/filename can be abbreviated to just their first letters: m, n, s, and f.

=head1 VERSION

You are running Paisley version 0.1a1, which was last modified on June 1, 2007.

=head1 LICENSE

This software is released under the terms of the GNU General Public License, version 2.0.

=head1 COPYRIGHT

This software is copyright (c) 2007 by Andrew Merenbach.

=head1 AUTHOR

=over 8

=item Andrew Merenbach

=item http://www.merenbach.com/

=item chutney@sonic.net

=back

=cut