#!/usr/bin/perl

# Fraction determines a rational quotient from two integers
#	out to a specified decimal precision.
# Use the '--help' option for usage, license, and copyright information.
# This software is copyright (c) 2007 Andrew Merenbach.

use strict;
use warnings;
#use diagnostics;

use Pod::Usage;
use Getopt::Long 2.34;

$main::VERSION = '0.1a1';

# prototypes
sub main();
sub divide($$$);
sub advance($$$$);
sub exit_with_error($);
sub test_input_for_errors($$$);

sub main() {
	my %opts = (
		'numerator' => 0,	# numerator
		'denominator' => 1,	# denominator
		'scale' => 0,	# scale
		'newline' => 0,	# whether to print newline after output
			# ('e' stands for 'enter'--which is, I admit, rather silly)
	);
	
	GetOptions(
		'n|numerator=i' => \$opts{'numerator'},
		'd|denominator=i' => \$opts{'denominator'},
		's|scale=i' => \$opts{'scale'},
	
		'e|newline' => \$opts{'newline'},
	
		'help|?' => sub { pod2usage(-verbose => 2) },
	);
	
	# determine command-line options
	my $n = $opts{'numerator'};
	my $d = $opts{'denominator'};
	my $s = $opts{'scale'};
	my $e = $opts{'newline'};
	
	# test input for errors
	my @errors = test_input_for_errors($n, $d, $s);
	print "*** $_\n" foreach (@errors);
	die "At least one error occurred; aborting...\n" if (@errors > 0);

	# perform division	
	my $quotient = divide($n, $d, $s);
	print $quotient;
	print "\n" if ($e == 1);
}

sub divide($$$) {
	my ($numerator, $denominator, $scale) = @_;
	my $quotient = "";
	
	my $fraction;
	
	# convert args to integers
	$numerator = int $numerator;
	$denominator = int $denominator;
	$scale = int $scale;
	
	# move along to the first digit
	advance(\$numerator, \$denominator, \$fraction, \$quotient);
	
	if ($scale > 0) {
		# no decimal point unless something follows
		$quotient .= '.';
		
		# move along to successive digits until scale is exhausted
		advance(\$numerator, \$denominator, \$fraction, \$quotient) while ($scale--);
	}

	return $quotient;
}

sub advance($$$$) {
	my ($numerator_ref, $denominator_ref, $fraction_ref, $quotient_ref) = @_;

	$$fraction_ref = int ($$numerator_ref / $$denominator_ref);
	$$quotient_ref .= $$fraction_ref;
	$$numerator_ref = 10 * ($$numerator_ref - $$fraction_ref * $$denominator_ref);
}

sub test_input_for_errors($$$) {
	my ($n, $d, $s) = @_;
	my @msgs;

	push @msgs, "Numerator must be an integer." if (int $n != $n);
	push @msgs, "Denominator must be an integer." if (int $d != $d);
	push @msgs, "Scale must be an integer." if (int $s != $s);

	push @msgs, "Numerator must be greater than or equal to zero." if ($n < 0);
	push @msgs, "Denominator must be greater than zero." if ($d <= 0);
 	push @msgs, "Scale must be greater than or equal to zero." if ($s < 0);
		
	return @msgs;
}

main();

__END__

=head1 NAME

Fraction - the extended quotient calculator

=head1 SYNOPSIS

fraction [-n numerator] [-d denominator] [-s scale] [-e]

=head1 OPTIONS

=over 8

=item -n <number>	the numerator (dividend); must be an integer

=item -d <number>	the denominator (divisor); must be an integer

=item -s <number>	the scale (decimal precision); must be an integer

=item -e 			print a newline after output

=item -?			display this help

=item --help		display this help

=back

=head1 DESCRIPTION

Fraction performs an extended long-division with a numerator, denominator, and scale (decimal precision) input by the user.

=head1 EXAMPLES

The following examples show how to use the software.

- To perform a simple division, use the following format:

	fraction -n 1 -d 7 -s 32

- To pipe the output into Paisley for pattern-searching, use:

	fraction -n 1 -d 7 -s 32 | paisley -r

=head1 VERSION

You are running Fraction version 0.1a1, which was last modified on May 28, 2007.

=head1 LICENSE

This software is released under the terms of the GNU General Public License, version 2.0.

=head1 COPYRIGHT

This software is copyright (c) 2007 by Andrew Merenbach.

=head1 AUTHOR

=over 8

=item Andrew Merenbach

=item http://www.merenbach.com/

=item chutney@sonic.net

=back

=cut